"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const workspace_tools_1 = require("workspace-tools");
const logger_1 = require("../logger");
function filterPackages(options) {
    const { since, scope, allPackages, deps, root, ignore } = options;
    let filtered = [];
    let hasScopes = Array.isArray(scope) && scope.length > 0;
    let hasSince = typeof since !== "undefined";
    // If NOTHING is specified, use all packages
    if (!hasScopes && !hasSince) {
        logger_1.logger.verbose("filterPackages", "scope: all packages");
        filtered = Object.keys(allPackages);
    }
    // If scoped is defined, get scoped packages
    if (hasScopes) {
        const scoped = workspace_tools_1.getScopedPackages(scope, allPackages);
        filtered = filtered.concat(scoped);
        logger_1.logger.verbose("filterPackages", `scope: ${scoped.join(",")}`);
    }
    if (hasSince) {
        const changedPackages = workspace_tools_1.getChangedPackages(root, since, ignore);
        filtered = filtered.concat(changedPackages);
        logger_1.logger.verbose("filterPackages", `changed: ${changedPackages.join(",")}`);
    }
    if (deps) {
        filtered = filtered.concat(workspace_tools_1.getTransitiveDependencies(filtered, allPackages));
    }
    const unique = new Set(filtered);
    return [...unique];
}
exports.filterPackages = filterPackages;
