"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const findNpmClient_1 = require("../workspace/findNpmClient");
const child_process_1 = require("child_process");
const logger_1 = require("../logger");
const taskWrapper_1 = require("./taskWrapper");
const path_1 = __importDefault(require("path"));
const p_queue_1 = __importDefault(require("p-queue"));
let npmCmd;
let queue;
let bail = false;
let activeProcesses = new Set();
function npmTask(task, info, config, context, root) {
    const { node, args, npmClient, concurrency } = config;
    const logger = logger_1.taskLogger(info.name, task);
    // cached npmCmd
    queue = queue || new p_queue_1.default({ concurrency });
    npmCmd = npmCmd || findNpmClient_1.findNpmClient(npmClient);
    const npmArgs = [...node, "run", task, "--", ...args];
    if (bail) {
        return Promise.reject();
    }
    return queue.add(() => taskWrapper_1.taskWrapper(info, task, () => new Promise((resolve, reject) => {
        if (!info.scripts || !info.scripts[task]) {
            logger.info(`Empty script detected, skipping`);
            return resolve();
        }
        logger.verbose(`Running ${[npmCmd, ...npmArgs].join(" ")}`);
        const cp = child_process_1.spawn(npmCmd, npmArgs, {
            cwd: path_1.default.dirname(info.packageJsonPath),
            stdio: "pipe",
            env: Object.assign(Object.assign({}, process.env), (process.stdout.isTTY && { FORCE_COLOR: "1" })),
        });
        activeProcesses.add(cp);
        const stdoutLogger = new logger_1.NpmLogWritable(info.name, task);
        cp.stdout.pipe(stdoutLogger);
        const stderrLogger = new logger_1.NpmLogWritable(info.name, task);
        cp.stderr.pipe(stderrLogger);
        cp.on("exit", handleChildProcessExit);
        function handleChildProcessExit(code) {
            if (code === 0) {
                activeProcesses.delete(cp);
                return resolve();
            }
            bail = true;
            cp.stdout.destroy();
            cp.stdin.destroy();
            reject();
        }
    }), config, context, root));
}
exports.npmTask = npmTask;
function killAllActiveProcesses() {
    for (const cp of activeProcesses) {
        cp.kill("SIGKILL");
    }
}
exports.killAllActiveProcesses = killAllActiveProcesses;
