"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const task_scheduler_1 = require("@microsoft/task-scheduler");
const npmTask_1 = require("./npmTask");
const filterPackages_1 = require("./filterPackages");
const logger_1 = require("../logger");
async function runTasks(options) {
    const { graph, workspace, context, config } = options;
    let pipeline = task_scheduler_1.createPipeline(graph, {
        // dummy logger for task-scheduler because lage already has logger for its tasks
        logger: {
            error: (_msg) => { },
            log: (_msg) => { },
        },
        exit: (code) => { },
    });
    const taskNames = Object.keys(config.pipeline);
    for (const [task, taskDeps] of Object.entries(config.pipeline)) {
        const deps = taskDeps.filter((dep) => !dep.startsWith("^"));
        const topoDeps = taskDeps
            .filter((dep) => dep.startsWith("^"))
            .map((dep) => dep.slice(1));
        pipeline = pipeline.addTask({
            name: task,
            deps,
            topoDeps,
            run: async (_location, _stdout, _stderr, pkg) => {
                const scripts = workspace.allPackages[pkg].scripts;
                if (scripts && scripts[task]) {
                    return (await npmTask_1.npmTask(task, workspace.allPackages[pkg], config, context, workspace.root));
                }
                return true;
            },
        });
    }
    // Filter packages per --scope and command(s)
    const filteredPackages = filterPackages_1.filterPackages({
        root: workspace.root,
        allPackages: workspace.allPackages,
        deps: config.deps,
        scope: config.scope,
        since: config.since,
        ignore: config.ignore,
    });
    // Set up the longest names of tasks and scripts for nice logging
    logger_1.setTaskLogMaxLengths(Object.keys(workspace.allPackages).reduce((l, pkg) => (l < pkg.length ? pkg.length : l), 0), taskNames.reduce((l, task) => (l < task.length ? task.length : l), 0));
    await pipeline.go({
        packages: filteredPackages,
        tasks: config.command,
    });
}
exports.runTasks = runTasks;
